@@ -1,19 +1,11 @@
-#!/usr/bin/python3
-
-'''
-SPDX-License-Identifier: Apache-2.0
-Copyright 2017 Massachusetts Institute of Technology.
-'''
-
 import sys
 
-from keylime import keylime_logging
-from keylime import tenant
+from keylime import keylime_logging, tenant
 
-logger = keylime_logging.init_logging('tenant')
+logger = keylime_logging.init_logging("tenant")
 
 
-def main():
+def main() -> None:
     try:
         tenant.main()
     except tenant.UserError as ue:
