@@ -5,13 +5,12 @@ Revises: a79c27ec1054
 Create Date: 2022-01-11 20:54:00.299250
 
 """
-from alembic import op
 import sqlalchemy as sa
-
+from alembic import op
 
 # revision identifiers, used by Alembic.
-revision = '9169f80345ed'
-down_revision = 'a79c27ec1054'
+revision = "9169f80345ed"
+down_revision = "a79c27ec1054"
 branch_labels = None
 depends_on = None
 
@@ -24,7 +23,6 @@ def downgrade(engine_name):
     globals()[f"downgrade_{engine_name}"]()
 
 
-
 def upgrade_registrar():
     pass
 
@@ -34,11 +32,11 @@ def downgrade_registrar():
 
 
 def upgrade_cloud_verifier():
-    with op.batch_alter_table('verifiermain') as batch_op:
+    with op.batch_alter_table("verifiermain") as batch_op:
         # Using server_default to assign old rows a valid value
-        batch_op.add_column(sa.Column('supported_version', sa.String(length=50), nullable=False, default="1.0"))
-        batch_op.alter_column('supported_version', server_default=None)
+        batch_op.add_column(sa.Column("supported_version", sa.String(length=50), nullable=False, default="1.0"))
+        batch_op.alter_column("supported_version", server_default=None)
 
 
 def downgrade_cloud_verifier():
-    op.drop_column('verifiermain', 'supported_version')
+    op.drop_column("verifiermain", "supported_version")
